#> BEGIN DESCRIPTION
#> Package: SVSI
#> Type: Package
#> Title: Set-Valued System Identification Approach to Identifying Rare Variants in Sequencing Studies for Binary/Ordered Categorical Outcome 
#> Version: 0.1
#> Author: Guolian Kang <guolian.kang@stjude.org>,  Wenjian Bi <biwenjian@amss.ac.cn>
#> Maintainer: Zhifa Liu <zhifa.liu@stjude.org>
#> Depends: R (>= 2.15.1), MASS, Rlab
#> Date: 2014-11-7
#> Description: Performs genetic association analysis for rare variants for binary/ordered categorical outcome.
#> License: GPL-2
#> LazyData: yes
#> LazyLoad: yes  
#> URL: http://www.stjuderesearch.org/site/depts/biostats/software
#> END DESCRIPTION
library(MASS)
library(Rlab) 
##############################################################exportPattern("^[[:alpha:]]+")
#> BEGIN NAMESPACE
#> exportPattern("^[[:alpha:]]+")
#> END NAMESPACE

#############################################################
#> BEGIN SVSI-package
#> \alias{SVSI-package}
#> \alias{SVSI}
#> \docType{package}
#> \title{This package defines routines to perform genetic association analysis for rare variants for binary and ordered categorical traits using SVSI approach.}
#> \description{Defines routines to perform genetic association analysis for rare variants for binary and ordered categorical traits using SVSI approach.}
#> \details{
#> \tabular{ll}{
#> Package: \tab SVSI \cr
#> Type: \tab Package\cr
#> Version: \tab 0.1\cr
#> Date: \tab 2014-10-28\cr
#> License: \tab GPL(>=2) \cr }
#> }
#> \author{Guolian Kang <guolian.kang@stjude.org> Wenjian Bi <biwenjian@amss.ac.cn>
#>
#> Maintainer: Zhifa Liu <zhifa.liu@stjude.org>
#> }
#> \references{Wenjian Bi, Guolian Kang, Yanlong Zhao, Yuehua Cui, Song Yan, Yun Li, Cheng Cheng, Stanley B. Pounds, Michael J. Borowitz, Mary V. Relling, Jun J. Yang, Ching-Hon Pui, Stephen P. Hunger,  Christine M Hartford, Wing Leung, Ji-Feng Zhang. SVSI: Fast and Powerful Set-Valued System Identification Approach to Identifying Rare Variants in Sequencing Studies for Ordered Categorical Traits. Annals of Human Genetics. 2014; Accepted with a major revision.
#>
#>
#> Guolian Kang, Wenjian Bi, Yanlong Zhao, Ji-Feng Zhang, Jun J Yang, Heng Xu, Mignon L Loh, Stephen P Hunger, Mary V Relling, Stanley Pounds, Cheng Cheng. A New System Identification Approach to Identify Genetic Variants in Sequencing Studies for a Binary Phenotype. Human Heredity. 78:104--116.}
#> END SVSI-package

#############################################################
#> BEGIN data_cate_geno
#> \alias{data_cate_geno}
#> \docType{data}
#> \title{An example data with the outcome of an ordered categorical phenotype and the genotype of SNPs}
#> \description{Gives the genotype and ordered categorical outcome data.}
#> \format{ A data frame with 697 observations for the following 26 variables:
#> \describe{
#>    \item{\code{FID}}{The first column is family ID.}
#>    \item{\code{IID}}{The second column is individual ID.}
#>    \item{\code{PAT}}{The third column is father ID.}
#>    \item{\code{MAT}}{The four column is mother ID.}
#>    \item{\code{SEX}}{The five column is individual ID.}
#>    \item{\code{status}}{The second column is ordered categorical outcome.}
#>    \item{\code{SNP1}}{SNP1 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP2}}{SNP2 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP3}}{SNP3 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP4}}{SNP4 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP5}}{SNP5 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP6}}{SNP6 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP7}}{SNP7 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP8}}{SNP8 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP9}}{SNP9 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP10}}{SNP10 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP11}}{SNP11 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP12}}{SNP12 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP13}}{SNP13 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP14}}{SNP14 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP15}}{SNP15 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP16}}{SNP16 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP17}}{SNP17 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP18}}{SNP18 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP19}}{SNP19 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP20}}{SNP20 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>  } }
#> \examples{data(data_cate_geno)}
#> \keyword{datasets}
#> END data_cate_geno

#############################################################
#> BEGIN data_cate_cova
#> \alias{data_cate_cova}
#> \docType{data}
#> \title{An example data with three covariates, two are binary and one is continuous.}
#> \description{Gives the covariates data that need to be adjusted for in the genetic association studies.}
#> \format{ A data frame with 697 observations for the following 4 variables:
#> \describe{
#>    \item{\code{FID}}{The first column is family ID.}
#>    \item{\code{IID}}{The second column is individual ID.}
#>    \item{\code{SEX}}{A binary covariate coded as 0 or 1.}
#>    \item{\code{AGE}}{A continuous covariate.}
#>    \item{\code{SMOKE}}{A binary covariate coded as 0 or 1.}
#>  } }
#> \examples{data(data_cate_cova)}
#> \keyword{datasets}
#> END data_cate_cova

#############################################################

#> BEGIN data_bina_geno
#> \alias{data_bina_geno}
#> \docType{data}
#> \title{An example data with the outcome of a binary outcome and the genotype of SNPs}
#> \description{Gives the genotype and the binary outcome data.}
#> \format{ A data frame with 697 observations for the following 26 variables:
#> \describe{
#>    \item{\code{FID}}{The first column is family ID.}
#>    \item{\code{IID}}{The second column is individual ID.}
#>    \item{\code{PAT}}{The third column is father ID.}
#>    \item{\code{MAT}}{The four column is mother ID.}
#>    \item{\code{SEX}}{The five column is individual ID.}
#>    \item{\code{status}}{The second column is ordered binagorical outcome.}
#>    \item{\code{SNP1}}{SNP1 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP2}}{SNP2 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP3}}{SNP3 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP4}}{SNP4 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP5}}{SNP5 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP6}}{SNP6 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP7}}{SNP7 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP8}}{SNP8 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP9}}{SNP9 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP10}}{SNP10 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP11}}{SNP11 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP12}}{SNP12 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP13}}{SNP13 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP14}}{SNP14 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP15}}{SNP15 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP16}}{SNP16 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP17}}{SNP17 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP18}}{SNP18 genotype data coded as 0, 1 or 2 at Null hypothesis.}
#>    \item{\code{SNP19}}{SNP19 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>    \item{\code{SNP20}}{SNP20 genotype data coded as 0, 1 or 2 at alternative hypothesis.}
#>  } }
#> \examples{data(data_bina_geno)}
#> \keyword{datasets}
#> END data_bina_geno

#############################################################
#> BEGIN data_bina_cova
#> \alias{data_bina_cova}
#> \docType{data}
#> \title{An example data with three covariates, two are binary and one is continuous.}
#> \description{Gives the covariates data that need to be adjusted for in the genetic association studies.}
#> \format{ A data frame with 697 observations for the following 4 variables:
#> \describe{
#>    \item{\code{FID}}{The first column is family ID.}
#>    \item{\code{IID}}{The second column is individual ID.}
#>    \item{\code{SEX}}{A binary covariate coded as 0 or 1.}
#>    \item{\code{AGE}}{A continuous covariate.}
#>    \item{\code{SMOKE}}{A binary covariate coded as 0 or 1.}
#>  } }
#> \examples{data(data_bina_cova)}
#> \keyword{datasets}
#> END data_bina_cova

################################################################################
#                                                                              #  
#                     SVSI method for the ordered categorical phenotype        #
#                                                                              #
################################################################################

################################################################################
#> BEGIN SVSI_OrderCate_Whole
#> \title{Perform whole-genome genetic association analyses for ordered categorical traits across the whole genome}
#> \description{Perform whole-genome genetic association analysis for ordered categorical traits using SVSI approach}
#> \arguments{
################################################################################
#
#                          SVSI method for the ordered categorical phenotype
#
#                    ----Main association part----
################################################################################
SVSI_OrderCate_Whole=function(data_cate_geno,    #> \item{data_cate_geno}{The ordered categorical and genotype data.}
                              data_cate_cova=NA, #> \item{data_cate_cova}{The covariate data including binary and continuous covariates. The default is NA.}
							  prefix="",         #> \item{prefix}{Prefix for the result file}  
                              result.dir=NULL)   #> \item{result.dir}{The working directory to store the results}   

#> } 
{status=data_cate_geno$status;
 n.snp=dim(data_cate_geno)[2]-6; 
 if (n.snp==1){  
              if(length(dim(data_cate_cova))==0){ 
              genotype=as.numeric(as.matrix(data_cate_geno[,7:(n.snp+6)]));
              out=SVSI_OrderCate(genotype,status=status,cova=NA); #apply(data_cate_geno[,7:(n.snp+6)],2,SVSI_OrderCate,status=status,cova=NA);print(out)
			  out=cbind(c("",names(data_cate_geno)[7:(n.snp+6)]),t(cbind(c("Variable","Estimate","SE","Stat","Pval"),out)));    
			  } else {
			  n.cov=dim(data_cate_cova)[2]-2;
			  genotype=as.numeric(as.matrix(data_cate_geno[,7:(n.snp+6)]));
			  out=SVSI_OrderCate(genotype,status=status,cova=data_cate_cova);#apply(data_cate_geno[,7:(n.snp+6)],2,SVSI_OrderCate,status=status,cova=data_cate_cova);
			  out=cbind(c("",names(data_cate_geno)[7:(n.snp+6)]),t(cbind(rep(c("Variable","Estimate","SE","Stat","Pval"),n.cov+1),out)));                              
				    }
             } else{
			 if(length(dim(data_cate_cova))==0){ 
			 genotype=data_cate_geno[,7:(n.snp+6)];
			 out=apply(data_cate_geno[,7:(n.snp+6)],2,SVSI_OrderCate,status=status,cova=NA);#print(out)
			 out=cbind(c("",names(data_cate_geno)[7:(n.snp+6)]),t(cbind(c("Variable","Estimate","SE","Stat","Pval"),out)));                  
             }else{
			 n.cov=dim(data_cate_cova)[2]-2;
			 genotype=data_cate_geno[,7:(n.snp+6)];
			 out=apply(data_cate_geno[,7:(n.snp+6)],2,SVSI_OrderCate,status=status,cova=data_cate_cova);
			 out=cbind(c("",names(data_cate_geno)[7:(n.snp+6)]),t(cbind(rep(c("Variable","Estimate","SE","Stat","Pval"),n.cov+1),out)));                              
				  }
}
svsi.res.file<-paste(result.dir,prefix,"svsi_cate_results.txt",sep="")
write.table(out,svsi.res.file,row.names=F,col.names=F,quote=T)
message(paste("SVSI analyses completed:",date()))
out
}
#>\examples{
#>       library("SVSI")
#>       data(data_cate_geno)
#>       data(data_cate_cova)
#>       res=SVSI_OrderCate_Whole(data_cate_geno,data_cate_cova)
#>}

#> END  SVSI_OrderCate_Whole


#> BEGIN SVSI_OrderCate
#> \title{Perform genetic association analysis for ordered categorical traits}
#> \description{Perform genetic association analysis for ordered categorical traits using SVSI approach}
#> \arguments{
SVSI_OrderCate=function(  geno,   #> \item{geno}{The genotype of one SNP with 0, 1 or 2.}
                        status,   #> \item{status}{The ordered categorical outcome.}
						  cova=NA)#> \item{cova}{The covariates. The default is NA.}
#> } 
{#
	status=as.matrix(status);
	snp.names=names(geno);
	if(length(dim(cova))==0){
							var.in=as.matrix(geno); 
							}else{
							var.in=as.matrix(cbind(geno,cova[,3:dim(cova)[2]])) 
								 }
	names.var=colnames(var.in);
	r=max(status);
	C=1:r;
	N=dim(var.in)[1];
	n=dim(var.in)[2];
	theta=rep(0,n);

	p1_inv=ginv(t(var.in)%*%var.in);
	for (i in 1:100){
	C.old=C;theta.old=theta;
	C=cDetect(C,var.in,status,theta);
	theta=tDetect(C,var.in,status,theta,p1_inv);
	if (max(norm(C-C.old,type="2"),norm(theta-theta.old,type="2"))<1e-8){ break;}
					}
	est.y=-var.in%*%theta;
	est.y.add=c();
	for (i in 1:r){
	est.y.add=cbind(est.y.add,est.y+C[i]);
				  }  	 
	sigma=1;
	fish.para=dnorm(est.y.add[,1],0,sigma)^2/pnorm(est.y.add[,1],0,sigma);
	for (i in 1:(r-1)){
					   fish.para=fish.para+(dnorm(est.y.add[,i+1],0,sigma)-dnorm(est.y.add[,i],0,sigma))^2/(pnorm(est.y.add[,i+1],0,sigma)-pnorm(est.y.add[,i],0,sigma));
					   }				    				
	fish.para=fish.para+dnorm(est.y.add[,r],0,sigma)^2/(1-pnorm(est.y.add[,r],0,sigma));
	fish.para=as.matrix(fish.para);
	inv.fish.matr=matrix(0,n+1,n+1);
	inf1=1e7;
	for (i in 1:N){
				   inv.fish.matr=inv.fish.matr+fish.para[i,1]*c(var.in[i,],-1)%*%t(c(var.in[i,],-1));
				   }
	if (sum(1*is.infinite(inv.fish.matr))>0){
											inv.fish.matr[inv.fish.matr==Inf]=inf1;
											inv.fish.matr[inv.fish.matr==-Inf]=-inf1;
											}
	if (sum(1*is.nan(inv.fish.matr))>0){
									   inv.fish.matr[is.nan(inv.fish.matr)]=inf1;
									   }
	fish.matr=ginv(inv.fish.matr);
	SE=sqrt(diag(fish.matr)[1:n]);
	Stat=theta^2/diag(fish.matr)[1:n];
	pval=pchisq(Stat,df=1, ncp = 0, lower.tail = FALSE);
	theta=signif(theta,4);
	SE=as.matrix(signif(SE,4));
	Stat=signif(Stat,4);
	pval=signif(pval,4);
	coeff=cbind.data.frame(theta,SE,Stat,pval);

	if (n==1){
			 coeff=c("geno",as.matrix(coeff))
			 }else{ 
			 coeff=cbind(names.var,coeff);
			 }
	return(c(t(coeff)))
}
#> END  SVSI_OrderCate
#SVSI_OrderCate(data.s)
################################################################################
#
#                          SVSI method
#
#                         ----T Step----
################################################################################

#> BEGIN tDetect
#> \title{estimate the association coefficients}
#> \description{estimate the association coefficients}
#> \arguments{
tDetect=function(   C,#> \item{C}{The threshold from C step.}
               var.in,#> \item{var.in}{The input variables obtained from C step. }
			   status,#> \item{status}{The status outcome data.}
			   theta, #> \item{theta}{The coefficient estimation from C step.}
			   p1_inv )#> \item{p1_inv}{The inverse matrix.}


#> } 
{
	r=dim(as.matrix(C))[1];
	n=dim(var.in)[2];
	N=dim(var.in)[1];
	sigma=1;
	phi=t(var.in);
	estTheta=rep(1,n)*theta;
	temp_p=t(phi)%*%estTheta;
	temp4p2=-1*dnorm(C[1]-temp_p,0,sigma)/pnorm(C[1]-temp_p,0,sigma)*(status==0);
	for (i in 1:(r-1)){
		temp4p2=temp4p2+(-1*dnorm(C[i+1]-temp_p,0,sigma)+dnorm(C[i]-temp_p,0,sigma))/(pnorm(C[i+1]-temp_p,0,sigma)-pnorm(C[i]-temp_p,0,sigma))*(status==i);
					  }
	temp4p2=temp4p2+dnorm(C[r]-temp_p,0,sigma)/(1-pnorm(C[r]-temp_p,0,sigma))*(status==r);
	p2=sigma^2*phi%*%temp4p2;
	temp=p1_inv%*%p2;
	theta=theta+temp;
	out=theta;
	return(out);
}
#> END tDetect
################################################################################
#
#                          SVSI method
#                        ----C Step----
################################################################################
#> BEGIN cDetect
#> \title{estimate the thresholds}
#> \description{estimate the thresholds}
#> \arguments{
cDetect=function(    C,#> \item{C}{The threshold from T step.}
                var.in,#> \item{var.in}{The input variables obtained from T step. }
				status,#> \item{status}{The status outcome data.}
				theta) #> \item{theta}{The coefficient estimation from T step.}
				#> } 
				{
	N=dim(var.in)[1];
	x=var.in%*%theta;
	r=dim(as.matrix(C))[1];
	C.X=array(0,c(N,r));
	for (i in 1:r){
	C.X[,i]=C[i]-x;
				  }
	Dl=array(0,r);
	Dl[1]=sum(dnorm(C.X[,1])/pnorm(C.X[,1])*(status==0)
			 -dnorm(C.X[,1])/(pnorm(C.X[,2])-pnorm(C.X[,1]))*(status==1));
	Dl[r]=sum(dnorm(C.X[,r])/(pnorm(C.X[,r])-pnorm(C.X[,r-1]))*(status==r-1)
			 -dnorm(C.X[,r])/(1-pnorm(C.X[,r]))*(status==r));
	if (r>2){
	for (i in 2:(r-1)){
	Dl[i]=sum(dnorm(C.X[,i])/(pnorm(C.X[,i])-pnorm(C.X[,i-1]))*(status==i-1)
			 -dnorm(C.X[,i])/(pnorm(C.X[,i+1])-pnorm(C.X[,i]))*(status==i));
					   }
			}
	DDl=array(0,c(r,r));
	DDl[1,1]=sum(((-dnorm(C.X[,1])*C.X[,1]*pnorm(C.X[,1])-dnorm(C.X[,1])^2)/pnorm(C.X[,1])^2)*(status==0)
				+((dnorm(C.X[,1])*C.X[,1]*(pnorm(C.X[,2])-pnorm(C.X[,1]))-dnorm(C.X[,1])^2)/(pnorm(C.X[,2])-pnorm(C.X[,1]))^2)*(status==1));
	DDl[r,r]=sum(((-dnorm(C.X[,r])*C.X[,r]*(pnorm(C.X[,r])-pnorm(C.X[,r-1]))-dnorm(C.X[,r])^2)/(pnorm(C.X[,r])-pnorm(C.X[,r-1]))^2)*(status==r-1)
				+((dnorm(C.X[,r])*C.X[,r]*(1-pnorm(C.X[,r]))-dnorm(C.X[,r])^2)/(1-pnorm(C.X[,r]))^2)*(status==r));
	if (r>2){
	for (i in 2:(r-1)){
	DDl[i,i]=sum(((-dnorm(C.X[,i])*C.X[,i]*(pnorm(C.X[,i])-pnorm(C.X[,i-1]))-dnorm(C.X[,i])^2)/(pnorm(C.X[,i])-pnorm(C.X[,i-1]))^2)*(status==i-1)
				+((dnorm(C.X[,i])*C.X[,i]*(pnorm(C.X[,i+1])-pnorm(C.X[,i]))-dnorm(C.X[,i])^2)/(pnorm(C.X[,i+1])-pnorm(C.X[,i]))^2)*(status==i));
					  }
			}
	for (i in 1:(r-1)){
	DDl[i,i+1]=sum((dnorm(C.X[,i+1])*dnorm(C.X[,i])/(pnorm(C.X[,i+1])-pnorm(C.X[,i]))^2)*(status==i));
	DDl[i+1,i]=DDl[i,i+1];
					  }
	C=C-ginv(DDl)%*%Dl;
	out=C;
	return(out)
}
#> END cDetect

#################################################################################
##                                                                              #
##                           END OF SV for the  Binary phenotype    #
##                                                                              #
#################################################################################
################################################################################
#> BEGIN SVSI_Bina_Whole
#> \title{Perform whole-genome genetic association analysis for binary traits}
#> \description{Perform whole-genome genetic association analysis for binary traits using SVSI approach}
#> \arguments{
################################################################################
#                                                                              # 
#                          SVSI method for the binary phenotype                #
#                                                                              #
#                    ----Main association part----                             #
################################################################################
SVSI_Bina_Whole=function(data_bina_geno, #> \item{data_bina_geno}{The binary trait and genotype data.}
                      data_bina_cova=NA, #> \item{data_bina_cova}{The covariate data including binary and continuous covariates. The default is NA.}
					  prefix="",         #> \item{prefix}{Prefix for the result file}  
                      result.dir=NULL)   #> \item{result.dir}{The working directory to store the results} 

#> } 
{
	status=data_bina_geno$status;
	 n.snp=dim(data_bina_geno)[2]-6;
	  if (n.snp==1){
	   if(length(dim(data_bina_cova))==0){ 
	 genotype=as.numeric(as.matrix(data_bina_geno[,7:(n.snp+6)]));
	 out=SVSI_Bina(genotype,status=status,cova=NA); #apply(data_bina_geno[,7:(n.snp+6)],2,SVSI_Orderbina,status=status,cova=NA);print(out)
	 out=cbind(c("",names(data_bina_geno)[7:(n.snp+6)]),t(cbind(c("Variable","Estimate","SE","Stat","Pval"),out)));                        
										}else{
	 n.cov=dim(data_bina_cova)[2]-2;
	 genotype=as.numeric(as.matrix(data_bina_geno[,7:(n.snp+6)]));
	out=SVSI_Bina(genotype,status=status,cova=data_bina_cova);#apply(data_bina_geno[,7:(n.snp+6)],2,SVSI_Orderbina,status=status,cova=data_bina_cova); 
	out=cbind(c("",names(data_bina_geno)[7:(n.snp+6)]),t(cbind(rep(c("Variable","Estimate","SE","Stat","Pval"),n.cov+1),out)));                              
								   }
	 } else { 
	 if(length(dim(data_bina_cova))==0){ 
	 genotype=data_bina_geno[,7:(n.snp+6)];
	 out=apply(data_bina_geno[,7:(n.snp+6)],2,SVSI_Bina,status=status,cova=NA);#print(out)
	 out=cbind(c("",names(data_bina_geno)[7:(n.snp+6)]),t(cbind(c("Variable","Estimate","SE","Stat","Pval"),out)));                  
	}else{
	 n.cov=dim(data_bina_cova)[2]-2;
	 genotype=data_bina_geno[,7:(n.snp+6)];
	 out=apply(data_bina_geno[,7:(n.snp+6)],2,SVSI_Bina,status=status,cova=data_bina_cova); 
	 out=cbind(c("",names(data_bina_geno)[7:(n.snp+6)]),t(cbind(rep(c("Variable","Estimate","SE","Stat","Pval"),n.cov+1),out)));                              
								   }
							}						
	svsi.res.file<-paste(result.dir,prefix,"svsi_bina_results.txt",sep="")
	write.table(out,svsi.res.file,row.names=F,col.names=F,quote=T)
	message(paste("SVSI analyses completed:",date()))
	out
}
#>\examples{
#>       library("SVSI")
#>       data(data_bina_geno)
#>       data(data_bina_cova)
#>       res=SVSI_Bina_Whole(data_bina_geno,data_bina_cova)
#>}

#> END  SVSI_Bina_Whole

#> BEGIN SVSI_Bina
#> \title{Perform genetic association analysis for the binary traits}
#> \description{Perform genetic association analysis for the binary traits using SVSI approach}
#> \arguments{
SVSI_Bina=function( geno,#> \item{geno}{The genotype of one SNP with 0, 1 or 2.}
                  status,#> \item{status}{The binary outcome.}
			     cova=NA)#> \item{cova}{The covariates. The default is NA.}
#> } 
{
	if (length(dim(cova))==0){geno=geno}else{geno=cbind(geno,cova[,3:dim(cova)[2]]);} 
	names.var=names(geno);
	geno=as.matrix(geno)
	N=dim(geno)[1];
	n=dim(geno)[2];
	out=QIFun(geno,pheno=status,iteration_for_estTheta=200);
	est_theta=out$est_theta;
	est_c    =out$est_c;
	est_a=est_c-geno%*%est_theta;
	sigma=1;
	inf1=1e7;
	fisher_parameter=1/(pnorm(est_a,0,sigma)*(1-pnorm(est_a,0,sigma)))*dnorm(est_a,0,sigma)^2;
	inv_fisher_matrix=matrix(0,n+1,n+1);
	for (i in 1:N){
		inv_fisher_matrix=inv_fisher_matrix+fisher_parameter[i,1]*c(geno[i,],-1)%*%t(c(geno[i,],-1));
				  }
	if (sum(1*is.infinite(inv_fisher_matrix))>0){
	inv_fisher_matrix[inv_fisher_matrix==Inf]=inf1;
	inv_fisher_matrix[inv_fisher_matrix==-Inf]=-inf1;
												 } 
	if (sum(1*is.nan(inv_fisher_matrix))>0){
	inv_fisher_matrix[is.nan(inv_fisher_matrix)]=inf1;
										   } 
	fisher_matrix=ginv(inv_fisher_matrix);

	SE=sqrt(diag(fisher_matrix)[1:n]);
	Stat=est_theta^2/diag(fisher_matrix)[1:n];
	pval=pchisq(Stat,df=1, ncp = 0, lower.tail = FALSE);
	theta=signif(est_theta,4);
	SE=as.matrix(signif(SE,4));
	Stat=signif(Stat,4);
	pval=signif(pval,4);

	coeff=cbind.data.frame(theta,SE,Stat,pval);

	if (n==1){coeff=c("geno",as.matrix(coeff))
			 }else{ 
	coeff=cbind(names.var,as.matrix(coeff));}
	return(c(t(coeff)))
}
#> END  SVSI_Bina


#> BEGIN QIFun
#> \title{SVSI algorithm to estimate all association coefficients}
#> \description{SVSI algorithm to estimate all association coefficients}
#> \arguments{
QIFun=function(  geno,                   #> \item{geno}{The genotype of one SNP with 0, 1 or 2 and covariates if available.} 
                pheno,                   #> \item{pheno}{The binary outcome.}
				iteration_for_estTheta)  #> \item{iteration_for_estTheta}{The number of iterations for estimation of association coefficients.}
#> } 
{
	n=dim(geno)[2];
	N=dim(geno)[1];# Number of individuals.
	matrix=t(geno);
	s=as.matrix(pheno);
	# "matrix" and "s" are input and output for every single SNP.

	################################################################################################################
	## QI process!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	################################################################################################################
	sigma=1;
	initialTheta=0;
	convcrit=1e-6;
	eps=2.2204e-16;
	seps=sqrt(eps);
	# The above is some set up for QI iteration.
	##
	#Part 2: The detailed precess of iteration.
	######################################
	phi=rbind(matrix,t(rep(-1,N)));
	estTheta=rep(1,n+1)*initialTheta;
	#The begin of itetarion.
	p1_inv=ginv(phi%*%t(phi));
	for (j in 1:iteration_for_estTheta){
		temp_p=-t(phi)%*%estTheta;
		temp4p2=sigma^2*dnorm(temp_p,0,sigma)/((-pnorm(temp_p,0,sigma))*(s)+(1-pnorm(temp_p,0,sigma))*(1-s));
		p2=phi%*%temp4p2;
		temp=p1_inv%*%p2;
		if (all(abs(temp)<convcrit*max(seps,estTheta))) {break;
		} else{
		estTheta=estTheta+temp;     
		}
	}
	est_c=estTheta[n+1,1];
	est_theta=estTheta[1:n,1];
	out=list(est_theta=est_theta,est_c=est_c);
	out
}
#> END  QIFun
#SVSI_Bina_Whole(data_bina_geno,data_bina_cova)

